@@ -1,3 +1,4 @@
+import logging
 import os
 import pathlib
 import sys
@@ -10,9 +11,14 @@ __all__ = ['ClamsApp']
 
 from typing import Union, Any, Optional
 
-from mmif import Mmif, Document, DocumentTypes, View, __specver__
+from mmif import Mmif, Document, DocumentTypes, View
 from clams.appmetadata import AppMetadata
 
+logging.basicConfig(
+    level=logging.WARNING,
+    format="%(asctime)s %(name)s %(levelname)-8s %(thread)d %(message)s",
+    datefmt="%Y-%m-%d %H:%M:%S")
+
 
 class ClamsApp(ABC):
     """
@@ -28,6 +34,9 @@ class ClamsApp(ABC):
             'description': 'The JSON body of the HTTP response will be re-formatted with 2-space indentation',
         },
     ]
+    # this key is used to store users' raw input params in the parameter dict 
+    # even after "refinement" (i.e., casting to proper data types)
+    _RAW_PARAMS_KEY = "#RAW#"
 
     def __init__(self):
         self.metadata: AppMetadata = self._load_appmetadata()
@@ -42,7 +51,8 @@ class ClamsApp(ABC):
             self.metadata_param_spec[param['name']] = (python_type[param['type']], param.get('multivalued', False))
         for param_spec in self.metadata.parameters:
             self.annotate_param_spec[param_spec.name] = (python_type[param_spec.type], param_spec.multivalued)
-
+        self.logger = logging.getLogger(self.metadata.identifier)
+        
     def appmetadata(self, **kwargs) -> str:
         """
         A public method to get metadata for this app as a string.
@@ -104,19 +114,20 @@ class ClamsApp(ABC):
         issued_warnings = []
         for key in runtime_params:
             if key not in self.annotate_param_spec:
-                issued_warnings.append(UserWarning(f'An undefined parameter {key} (value: {runtime_params[key]}) is passed'))
+                issued_warnings.append(UserWarning(f'An undefined parameter "{key}" (value: "{runtime_params[key]}") is passed'))
+        refined_params = self._refine_params(**runtime_params)
         with warnings.catch_warnings(record=True) as ws:
-            annotated = self._annotate(mmif, **runtime_params)
+            annotated = self._annotate(mmif, **refined_params)
             if ws:
                 issued_warnings.extend(ws)
         if issued_warnings:
             warnings_view = annotated.new_view()
-            self.sign_view(warnings_view)
+            self.sign_view(warnings_view, refined_params)
             warnings_view.metadata.warnings = issued_warnings
-        return annotated.serialize(pretty=pretty)
+        return annotated.serialize(pretty=pretty, sanitize=True)
 
     @abstractmethod
-    def _annotate(self, mmif: Mmif, **runtime_params) -> Mmif:
+    def _annotate(self, mmif: Mmif, _raw_parameters=None, **refined_parameters) -> Mmif:
         """
         An abstract method to generate (or load if stored elsewhere) the app 
         metadata at runtime. All CLAMS app must implement this.
@@ -126,7 +137,6 @@ class ClamsApp(ABC):
         
         #. Create a new view (or views) by calling :meth:`~mmif.serialize.mmif.Mmif.new_view` on the input mmif object.
         #. Call :meth:`~clams.app.ClamsApp.sign_view` with the input runtime parameters for the record.
-        #. Call :meth:`~clams.app.ClamsApp.get_configuration` to get an "upgraded" runtime parameters with default values.
         #. Call :meth:`~mmif.serialize.view.View.new_contain` on the new view object with any annotation properties specified by the configuration.
         #. Process the data and create :class:`~mmif.serialize.annotation.Annotation` objects and add them to the new view. 
         #. While doing so, get help from :class:`~mmif.vocabulary.document_types.DocumentTypes`, :class:`~mmif.vocabulary.annotation_types.AnnotationTypes` classes to generate ``@type`` strings.
@@ -138,7 +148,7 @@ class ClamsApp(ABC):
         """
         raise NotImplementedError()
     
-    def get_configuration(self, **runtime_params):
+    def _refine_params(self, **runtime_params):
         """
         Method to "fill" the parameter dictionary with default values, when a key-value is not specified in the input.
         The input map is not really "filled" as a copy of it is returned with addition of default values. 
@@ -146,6 +156,9 @@ class ClamsApp(ABC):
         :return: a copy of parameter map, with default values added
         :raises ValueError: when a value for a required parameter is not found in the input
         """
+        if self._RAW_PARAMS_KEY in runtime_params:
+            # meaning the dict is already refined, just return it 
+            return runtime_params
         conf = {}
         for parameter in self.metadata.parameters:
             if parameter.name in runtime_params:
@@ -156,7 +169,16 @@ class ClamsApp(ABC):
                 conf[parameter.name] = parameter.default
             else:
                 raise ValueError(f"Cannot find configuration for a required parameter \"{parameter.name}\".")
+        # raw input params are hidden under a special key
+        conf[self._RAW_PARAMS_KEY] = runtime_params
         return conf
+    
+    def get_configuration(self, **runtime_params):
+        warnings.warn("ClamsApp.get_configuration() is deprecated. "
+                      "If you are using this method in `_annotate()` method,"
+                      "it is no longer needed since `clams-python>1.0.9`.", 
+                      DeprecationWarning, stacklevel=2)
+        return self._refine_params(**runtime_params)
 
     def sign_view(self, view: View, runtime_conf: Optional[dict] = None) -> None:
         """
@@ -169,9 +191,19 @@ class ClamsApp(ABC):
         :param view: a view to sign
         :param runtime_conf: runtime configuration of the app as k-v pairs
         """
+        # TODO (krim @ 8/2/23): once all devs understood this change, make runtime_conf a required argument
+        if runtime_conf is None:
+            warnings.warn("`runtime_conf` argument for ClamsApp.sign_view() will "
+                          "no longer be optional in the future. Please pass `runtime_params` "
+                          "from _annotate() method.",
+                          FutureWarning, stacklevel=2)
         view.metadata.app = self.metadata.identifier
         if runtime_conf is not None:
-            view.metadata.add_parameters(**{k: str(v) for k, v in runtime_conf.items()})
+            if self._RAW_PARAMS_KEY in runtime_conf:
+                conf = runtime_conf[self._RAW_PARAMS_KEY]
+            else:
+                conf = runtime_conf
+            view.metadata.add_parameters(**{k: str(v) for k, v in conf.items()})
         
     def set_error_view(self, mmif: Union[str, dict, Mmif], runtime_conf: Optional[dict] = None) -> Mmif:
         """
