@@ -12,7 +12,7 @@ from mmif import vocabulary
 unresolved_app_version_num = 'unresolvable'
 app_version_envvar_key = 'CLAMS_APP_VERSION'
 # type aliases to use in app metadata and runtime parameter processing 
-primitives = Union[int, float, bool, str]
+real_valued_primitives = Union[int, float, bool, str]
 # these names are taken from the JSON schema data types
 param_value_types = Literal['integer', 'number', 'string', 'boolean']
 
@@ -38,12 +38,8 @@ def generate_app_version(cwd=None):
     gitcmd = shutil.which('git') 
     gitdir = (Path(sys.modules["__main__"].__file__).parent.resolve() if cwd is None else Path(cwd)) / '.git'
     if gitcmd is not None and gitdir.exists():
-        proc = subprocess.run(
-            f'{gitcmd} --git-dir "{gitdir}" describe --tags --always',
-            capture_output=True,
-            shell=True,
-            check=True,
-        )
+        proc = subprocess.run([gitcmd, '--git-dir', str(gitdir), 'describe', '--tags', '--always'], 
+                              capture_output=True, check=True)
         return proc.stdout.decode('utf8').strip()
     elif app_version_envvar_key in os.environ:
         return os.environ[app_version_envvar_key]
@@ -66,14 +62,21 @@ class _BaseModel(pydantic.BaseModel):
 
 class Output(_BaseModel):
     """
-    Defines a data model that describes output specification of a CLAMS app
+    Data model that describes output specification of a CLAMS app. 
+    
+    CLAMS apps are expected to have at least one output type, and each type must
+    be defined by a ``@type`` URI string. If the type has specific properties and values, 
+    they can be described in the (optional) ``properties`` field. Finally, a human-readable 
+    verbose description can be provided in the (optional) ``description`` field for users.
+    
+    Developers should take diligent care to include all output types and their properties in the app metadata. 
     """
     at_type: pydantic.AnyHttpUrl = pydantic.Field(
         ..., 
         alias="@type", 
         description="The type of the object. Must be a IRI string."
     )
-    properties: Dict[str, str] = pydantic.Field(
+    properties: Dict[str, real_valued_primitives] = pydantic.Field(
         {}, 
         description="(optional) Specification for type properties, if any."
     )
@@ -92,7 +95,14 @@ class Output(_BaseModel):
                 
 class Input(Output):
     """
-    Defines a data model that describes input specification of a CLAMS app
+    Data model that describes input specification of a CLAMS app. 
+    
+    CLAMS apps are expected to have at least one output type, and each type must
+    be defined by a ``@type`` URI string. If the type has specific properties and values, 
+    they can be described in the (optional) ``properties`` field. Finally, a human-readable 
+    verbose description can be provided in the (optional) ``description`` field for users.
+    
+    Developers should take diligent care to include all output types and their properties in the app metadata. 
     """
     required: bool = pydantic.Field(
         None, 
@@ -130,11 +140,11 @@ class RuntimeParameter(_BaseModel):
         ...,
         description=f"Type of the parameter value the app expects. Must be one of {param_value_types_values}."
     ) 
-    choices: List[primitives] = pydantic.Field(
+    choices: List[real_valued_primitives] = pydantic.Field(
         None, 
         description="(optional) List of string values that can be accepted."
     )
-    default: primitives = pydantic.Field(
+    default: real_valued_primitives = pydantic.Field(
         None, 
         description="(optional) Default value for the parameter. Only valid for optional parameters. Namely, setting "
                     "a default value makes a parameter 'optional'."
@@ -157,7 +167,7 @@ class RuntimeParameter(_BaseModel):
         
 class AppMetadata(pydantic.BaseModel):
     """
-    Defines a data model that describes a CLAMS app. 
+    Data model that describes a CLAMS app. 
     
     Can be initialized by simply passing all required key-value pairs. 
     
@@ -348,14 +358,19 @@ class AppMetadata(pydantic.BaseModel):
             raise ValueError(f"Cannot add a duplicate output '{new}'.")
 
     def add_parameter(self, name: str, description: str, type: param_value_types,
-                      choices: Optional[List[primitives]] = None,
+                      choices: Optional[List[real_valued_primitives]] = None,
                       multivalued: bool = False,
-                      default: primitives = None):
+                      default: real_valued_primitives = None):
         """
         Helper method to add an element to the ``parameters`` list. 
         """
+        # casting default values (when the value is not nothing) makes sure 
+        # the values are correctly casted by the pydantic
+        # see https://docs.pydantic.dev/1.10/usage/types/#unions
+        # e.g. casting 0.1 using the `primitives` dict will result in  0 (int)
+        # while casting "0.1" using the `primitives` dict will result in  0.1 (float)
         new_param = RuntimeParameter(name=name, description=description, type=type,
-                                     choices=choices, default=default, multivalued=multivalued)
+                                     choices=choices, default=str(default) if default else default, multivalued=multivalued)
         if new_param.name not in [param.name for param in self.parameters]:
             self.parameters.append(new_param)
         else:
