@@ -6,12 +6,12 @@ from flask_restful import Resource, Api
 from mmif import Mmif
 
 from clams.app import ClamsApp
-from clams.appmetadata import primitives
+from clams.appmetadata import real_valued_primitives
 
 
 class Restifier(object):
     """
-    Resitifier is a simple wrapper that takes a :class:`.ClamsApp` object and
+    Restifier is a simple wrapper that takes a :class:`.ClamsApp` object and
     turn it into a flaks-based HTTP server. For mapping between Python API and
     HTTP API, it relies on :class:`.ClamsHTTPApi` class.
 
@@ -61,6 +61,9 @@ class Restifier(object):
                     'bind': f'{host}:{port}',
                     'workers': number_of_workers(),
                     'threads': 2,
+                    # because the default is 'None'
+                    'accesslog': '-',
+                    # errorlog, however, is redirected to stderr by default since 19.2, so no need to set
                 }
                 self.options.update(options)
                 self.application = app
@@ -157,10 +160,10 @@ class ParameterCaster(object):
 
     :param param_spec: A specification of a data types of parameters
     """
-    def __init__(self, param_spec: Dict[str, Tuple[primitives, bool]]):
+    def __init__(self, param_spec: Dict[str, Tuple[real_valued_primitives, bool]]):
         self.param_spec = param_spec
 
-    def cast(self, args: Dict[str, List[str]]) -> Dict[str, Union[primitives, List[primitives]]]:
+    def cast(self, args: Dict[str, List[str]]) -> Dict[str, Union[real_valued_primitives, List[real_valued_primitives]]]:
         """
         Given parameter specification, tries to cast values of args to specified Python data types.
         Note that this caster deals with query strings, thus all keys and values in the input args are plain strings. 
@@ -174,8 +177,8 @@ class ParameterCaster(object):
         """
         casted = {}
         for k, vs in args.items():
-            for v in vs:
-                if k in self.param_spec:
+            if k in self.param_spec:
+                for v in vs:
                     type_, multivalued = self.param_spec[k]
                     if multivalued or k not in casted:  # effectively only keeps the first value for non-multi params
                         if type_ == bool:
@@ -186,12 +189,15 @@ class ParameterCaster(object):
                             v = self.int_param(v)
                         elif type_ == str:
                             v = self.str_param(v)
-                        if not multivalued:
-                            casted[k] = v
-                        else:
+                        if multivalued:
                             casted.setdefault(k, []).append(v)
+                        else: 
+                            casted[k] = v
+            else:
+                if len(vs) > 1:
+                    casted[k] = vs
                 else:
-                    casted[k] = v[0]  # just keep the first value
+                    casted[k] = vs[0]
                     
         return casted
 
