@@ -5,9 +5,9 @@ import sys
 import tempfile
 import unittest
 import warnings
-from pathlib import Path
 from typing import Union
 
+import jsonschema
 import pytest
 from mmif import Mmif, Document, DocumentTypes, AnnotationTypes, View, __specver__
 
@@ -127,7 +127,7 @@ class TestClamsApp(unittest.TestCase):
         self.app.metadata.add_output(AnnotationTypes.BoundingBox, boxType='text')
         metadata = json.loads(self.app.appmetadata())
         self.assertEqual(len(metadata['output']), 2)
-        # these should not be added as a duplicate
+        # these should not be added because they are duplicates
         with self.assertRaises(ValueError):
             self.app.metadata.add_input(at_type=DocumentTypes.TextDocument)
         with self.assertRaises(ValueError):
@@ -202,11 +202,32 @@ class TestClamsApp(unittest.TestCase):
         self.assertIsNotNone(out_mmif)
         out_mmif = Mmif(out_mmif)
         self.assertEqual(len(out_mmif.views), 2)
+        for v in out_mmif.views:
+            if v.metadata.app == self.app.metadata.identifier:
+                self.assertEqual(len(v.metadata.parameters), 0)  # no params were passed when `annotate()` was called
+        out_mmif = self.app.annotate(self.in_mmif, pretty=False)
+        out_mmif = Mmif(out_mmif)
+        for v in out_mmif.views:
+            if v.metadata.app == self.app.metadata.identifier:
+                self.assertEqual(len(v.metadata.parameters), 1)  # 'pretty` parameter was passed 
         out_mmif = Mmif(self.app.annotate(out_mmif))
         self.assertEqual(len(out_mmif.views), 4)
         views = list(out_mmif.views)
         # insertion order is kept
         self.assertTrue(views[0].metadata.timestamp < views[1].metadata.timestamp)
+    
+    def test_annotate_returns_invalid_mmif(self):
+        m = Mmif(self.in_mmif)
+        v = m.new_view()
+        v.new_contain(AnnotationTypes.TimeFrame)
+        v.new_annotation(AnnotationTypes.TimeFrame, start=10, end=30)
+        # still, this view is not "signed"
+        from unittest.mock import MagicMock
+        self.app._annotate = MagicMock(return_value=m)
+        with self.assertRaises(jsonschema.ValidationError):
+            self.app.annotate(self.in_mmif)
+        
+        
 
     def test_open_document_location(self):
         mmif = ExampleInputMMIF.get_rawmmif()
@@ -219,25 +240,33 @@ class TestClamsApp(unittest.TestCase):
         with self.app.open_document_location(mmif.documents['i1'], Image.open) as f:
             self.assertEqual(f.size, (200, 71))
             
-    def test_get_configuration(self):
+    def test_refine_parameters(self):
         self.app.metadata.parameters = []
         self.app.metadata.add_parameter('param1', 'first_param', 'string')
         self.app.metadata.add_parameter('param2', 'second_param', 'string', default='second_default')
         self.app.metadata.add_parameter('param3', 'third_param', 'boolean', default='f')
         self.app.metadata.add_parameter('param4', 'fourth_param', 'integer', default='1', choices="1 2 3".split())
-        conf = self.app.get_configuration(param1='okay', non_parameter='should be ignored')
-        self.assertEqual(len(conf), 4)
+        self.app.metadata.add_parameter('param5', 'fifth_param', 'number', default='0.5')
+        dummy_params = {'param1': 'okay', 'non_parameter': 'should be ignored'}
+        conf = self.app._refine_params(**dummy_params)
+        # should not refine what's already refined
+        double_refine = self.app._refine_params(**conf)
+        self.assertTrue(clams.ClamsApp._RAW_PARAMS_KEY in double_refine)
+        self.assertEqual(double_refine, conf)
+        conf.pop(clams.ClamsApp._RAW_PARAMS_KEY, None)
+        self.assertEqual(len(conf), 5)  # 1 from `param1`, 4 from default value
         self.assertFalse('non_parameter' in conf)
         self.assertEqual(type(conf['param1']), str)
         self.assertEqual(type(conf['param2']), str)
         self.assertEqual(type(conf['param3']), bool)
         self.assertEqual(type(conf['param4']), int)
+        self.assertEqual(type(conf['param5']), float)
         with self.assertRaises(ValueError):
             # because param1 doesn't have a default value and thus a required param
-            self.app.get_configuration(param2='okay')
+            self.app._refine_params(param2='okay')
         with self.assertRaisesRegexp(ValueError, r'.+must be one of.+'):
             # because param4 can't be 4, note that param1 is "required" 
-            self.app.get_configuration(param1='p1', param4=4)
+            self.app._refine_params(param1='p1', param4=4)
             
     def test_error_handling(self):
         params = {'raise_error': True, 'pretty': True}
@@ -359,7 +388,9 @@ class TestParameterCaster(unittest.TestCase):
             'number_param': ["1.11"], 
             'int_param': [str(sys.maxsize)], 
             'bool_param': ['true'],
-            'str_multi_param': ['value1', 'value2']
+            'str_multi_param': ['value1', 'value2'],
+            'undefined_param_single': ['undefined_value1'],
+            'undefined_param_multi': ['undefined_value1', 'undefined_value2'],
         }
         self.assertTrue(all(map(lambda x: isinstance(x, str), itertools.chain.from_iterable(params.values()))))
         casted = caster.cast(params)
@@ -369,6 +400,10 @@ class TestParameterCaster(unittest.TestCase):
         self.assertTrue(isinstance(casted['int_param'], int))
         self.assertTrue(casted['bool_param'])
         self.assertEqual(set(casted['str_multi_param']), set(params['str_multi_param']))
+        self.assertTrue('undefined_param_single' in casted)
+        self.assertTrue('undefined_param_multi' in casted)
+        self.assertFalse(isinstance(casted['undefined_param_single'], list))
+        self.assertTrue(isinstance(casted['undefined_param_multi'], list))
         unknown_param_key = 'unknown'
         unknown_param_val = 'dunno'
         params[unknown_param_key] = [unknown_param_val]
